# -*- coding: utf-8 -*-
###########################################################################
#
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
# eole@ac-dijon.fr
#
# librairie pour la ompilation des paquets Eole
#
###########################################################################

from eolepack.configserver import EOLEPACK_FILE, COMPIL_PATH, SVN_ORIG_PATH, DEBEMAIL, DEBFULLNAME, CLONE_DIR
import os
from os.path import join, isdir, isfile
from shutil import copyfile, rmtree
from glob import glob
import subprocess
from eolepack import eolepackgit
from eolepack.errors import *

class Build:
    """actions de compilation"""
    def __init__(self, git_project, project, branch='master', distribution='', revision='', rep=''):
        """
        vars: variables des différents fichiers de config
        """
        self.project = project
        self.build_dir = COMPIL_PATH
        self.branch = branch
        self.distribution = distribution
        self.revision = revision
        self.rep = rep
        self.git_project = git_project
        self.git_path = join(CLONE_DIR, git_project)
        self.check_up_to_date()

    def check_clone(self):
        """ vérifie si le clone du projet existe, ainsi que la branche upstream
        """
        print "Clone du projet ", self.git_path
        if not os.path.isdir(join(self.git_path, '.git')):
            if os.path.isdir(self.git_path):
                print "Purge ancien dossier ", self.git_path
                rmtree(self.git_path)
            try:
                eolepackgit.git_clone(self.git_project)
            except:
                print "!! Erreur clone : "
                import traceback
                traceback.print_exc()
                raise BuildError, 'Erreur de création de clonage du projet %s' % self.git_project
        # vérification de la branche upstream
        try:
            branches = eolepackgit.git_branch(self.git_project)
            if 'upstream' not in [branche.name for branche in branches]:
                print "Ajout de la branche upstream pour ", self.git_project
                eolepackgit.git_new_branch(self.git_project, 'upstream')
        except:
            import traceback
            traceback.print_exc()
            raise BuildError, "Erreur à l'ajout de la branche upstream"

    def check_up_to_date(self):
        """ vérifie que le dépot local du projet est à jour
        """
        self.check_clone()
        eolepackgit.git_pull(self.git_path)

    def insert_debchange(self, long_release):
        if not isfile('debian/changelog'):
            raise ChangelogNotFound
        environ = {}
        environ['DEBEMAIL'] = DEBEMAIL
        environ['DEBFULLNAME'] = DEBFULLNAME
        retcode = subprocess.call('cd %s && /usr/bin/git-dch -a -R --distrib="%s" -N %s' \
                % (self.git_path, self.distribution, long_release), shell=True, env=environ)
        if retcode != 0:
            raise DebChangeError
        # commit du changelog
        eolepackgit.git_commit("Nouveau paquet : %s (%s) %s"% (self.project, long_release, self.distribution), self.git_path)

    def debuild(self, user='', passwd=''):
        """ options
        --git-export=<commit-id>
        --git-upstream-branch=branch_name
        --git-debian-branch=branch_name
        """
        options = '--git-export-dir=%s' % join(self.build_dir, self.git_project)
        if self.branch:
            options += ' --git-upstream-branch=%s --git-debian-branch=%s' % (self.branch, self.branch)
        if self.revision:
            options += ' --git-export=%s' % self.revision
        posttag_options = ""
        if user and passwd:
            posttag_options += """--git-posttag="git push --all && git push --tags" """
        build_cmd = "rm -rf %s && cd %s && /usr/bin/git-buildpackage %s --git-tag %s | tee -a %s/rapport_%s.txt" \
% (join(COMPIL_PATH, self.git_project, '*'), self.git_path, options, posttag_options, self.build_dir, self.project)
        retcode = 1
        try:
            retcode = subprocess.call(build_cmd, shell=True)
        except:
            print "Erreur de git-buildpackage :"
        if retcode != 0:
            raise BuildError
        if glob('%s/*.deb' % join(self.build_dir, self.project)) == []:
            # aucun paquet généré
            raise BuildError

    def put_deb_repository(self, repository='eole'):
        """
        envoi des paquets sur le repository
        """
        print "Envoi des paquets (dput)"
        os.system("""cd %s && dput %s *.changes"""% (join(self.build_dir, self.project), repository))

