# -*- coding: utf-8 -*-

###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# eolepacksvn : librairie d'accès à Subversion
#
###########################################################################

from eolepack import configserver
from eolepack import eolepackdb
from os.path import join
import os
import subprocess, tempfile, traceback, git

clone_dir = configserver.CLONE_DIR

def git_command(args, project):
    try:
        if args[0] in ['branch', 'clone', 'pull']:
            cmdargs = ['/usr/bin/git']
        else:
            cmdargs = ['/usr/bin/git', '--work-tree', join(clone_dir, project)]
        cmdargs.extend(args)
        print "lancement commande git (depuis %s) : %s" % (os.getcwd(), " ".join(cmdargs))
        gitcmd = subprocess.Popen(cmdargs, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        ret = gitcmd.wait()
        stdoutdata, stderrdata = gitcmd.communicate()
        return ret, stdoutdata, stderrdata
    except:
        traceback.print_exc()
        return None

def git_clone(project):
    """ initialisation du repository git """
    try:
        os.chdir(clone_dir)
        #git_url = join(configserver.GIT_ORIG_PATH, project)
        ret, out, err = git_command(('clone', '-q', configserver.GIT_ORIG_PATH+':'+project), project)
        assert ret == 0
        return join(clone_dir, project)
    except:
        traceback.print_exc()
        return None

def git_pull(project):
    """ mise à jour du repository git """
    try:
        os.chdir(join(clone_dir, project))
        ret, out, err = git_command(('pull',), project)
        assert ret == 0
        return True
    except:
        traceback.print_exc()
        return None

def git_new_branch(project, new_branch):
    """crée une nouvelle branche"""
    try:
        os.chdir(join(clone_dir, project))
        ret, out, err = git_command(('branch', new_branch), project)
        assert ret == 0
        return True
    except:
        traceback.print_exc()
        return None

def git_branch(project):
    """retourne les branches d'un projet"""
    try:
        repo = git.Repo(join(clone_dir,project))
        branches = repo.branches
        return branches
    except:
        traceback.print_exc()
        return None

def git_commit(msg, project):
    """commit dans le projet git"""
    try:
        os.chdir(join(clone_dir, project))
        ret, out, err = git_command(('commit', '-a', '-m', msg), project)
        assert ret == 0
        return out
    except:
        traceback.print_exc()
        return None

def git_push(git_url, user, password, project):
    """Push des modifications sur le serveur"""
    try:
        pass
    except:
        pass

def git_log(project, limit):
    """ Affichage du log git """
    try:
        repo = git.Repo(join(clone_dir,project))
        commits = repo.log()
        return commits[:limit]
    except:
        traceback.print_exc()
        return None

def git_tag(project):
    """ Affichage du tag git """
    try:
        repo = git.Repo(join(clone_dir,project))
        tags = repo.tags
        return tags
    except:
        traceback.print_exc()
        return None

