# -*- coding: utf-8 -*-

###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# pack.py : librairie pour la génération des paquets EoleNg
#
###########################################################################

from eolepack.configserver import EOLEPACK_FILE, COMPIL_PATH, CLONE_DIR
from eolepack.build import Build
from eolepack.errors import *
#from eolepack.cfgparser import Config
from eolepack import eolepackdb
from eolepack import tools
from time import time
from os import chdir, mkdir, system
from os.path import join


def pack(paquet, version, distribution, tags=None,
         branches=None, revision=None, epoch=None, changelog=None):
    """
    toutes les étapes pour la création d'un paquet
    """

    dist_ubuntu = eolepackdb.get_ubuntu_from_dist(distribution)

    # suppression des éventuelles compilations précédentes
    old_dir = join(COMPIL_PATH, "%s-%s-*" % (paquet, version))
    system("rm -rf %s" % old_dir)

    # répertoire de compilation spécifique
    compil_path = join(COMPIL_PATH, "%s-%s-%d"%(paquet, version, time()))
    mkdir(compil_path)
    chdir(compil_path)

    # récupération des infos dans la base de données
    try:
        id_paquet = eolepackdb.get_id_paquet(paquet, version, dist_ubuntu)
    except UnknownPackage:
        return False, "Paquet %s-%s inconnu" % (paquet, version)

    # FIXME : pas sûr que la base soit à jour !
    #try:
    #    last_release = eolepackdb.get_last_release(id_paquet)
    #    release = last_release+1
    #except LastReleaseNotFound:
    #    return False, "Il n'y a aucune release du paquet %s-%s\ndans la base de données"%(paquet,version)

    #configserver = Config(EOLEPACK_FILE)
    # Warning
    if tags:
        revision = tags

    # mise à jour du changelog


    # Instanciation de l'objet Build
    build = Build(paquet, branches, distribution, revision)

    # détection des sales caractères au plus tôt
    #try:
    #    print changelog
    #    changelog = tools.clean_log(changelog)
    #except UnicodeEncodeError:
    #    return False, "Problème d'encodage dans le changelog"
    # on rentre dans le projet
    chdir(join(CLONE_DIR, paquet))

    try:
        # Récupération du dernier numéro de release (HUM, HUM)
        release = int(file("debian/changelog").read().split()[1].split('eole')[1][0:-1])+1
    except:
        return False, "Erreur lors de la récupération\ndu dernier numéro de release !"

    # édition changelog
    long_release = "%s-eole%d" % (version, release)
    if epoch:
        long_release = "%s:%s" % (epoch, long_release)
    # compilation du paquet
    try:
        build.insert_debchange(long_release)
    except ChangelogNotFound:
        return False, "Fichier changelog non trouvé"
    except DebChangeError:
        return False, "Erreur lors du debchange"
    try:
        build.debuild()
    except BuildError:
        return False, "Erreur lors de la compilation du paquet\nVeuillez consulter le rapport ##FIXME##"

    # envoi sur le répository
    # build.put_deb_repository(repository='eole')

    return True, "compilation réussie :\n%s-%s" % (paquet, long_release)
